// Custom pipeline component for performing codeword swaps
// (C) 2003 - 2004, Que Press
namespace EncodeComponent
{
   using System;
   using System.Resources;
   using System.Drawing;
   using System.Collections;
   using System.Reflection;
   using System.ComponentModel;
   using System.Text;
   using System.IO;
   using Microsoft.BizTalk.Message.Interop;
   using Microsoft.BizTalk.Component.Interop;
   using System.Xml;
   using System.Xml.XPath;
	/// <summary>
	/// Implements a custom pipeline component for replacing an activity with a code word
	/// </summary>
	[ComponentCategory(CategoryTypes.CATID_PipelineComponent)]
   [ComponentCategory(CategoryTypes.CATID_Decoder)]
   [System.Runtime.InteropServices.Guid("ACC3F15A-C389-4a5d-8F8E-2A951CDC4C19")]
   public class EncodeLocation : 
      IBaseComponent, 
      Microsoft.BizTalk.Component.Interop.IComponent, 
      Microsoft.BizTalk.Component.Interop.IPersistPropertyBag, 
      IComponentUI
	{
      #region EncodeLocation Members
      private string connectString = null;
      static ResourceManager resManager = new ResourceManager("EncodeComponent.EncodeLocation", Assembly.GetExecutingAssembly());
      
      /// <summary>
      /// Public property will be exposed as a custom design-time property as long as accessors are provided
      /// </summary>
      /// 
      public string ConnectionString
      {
         get {return connectString;}
         set {connectString = value;}
      }
      #endregion

      #region IBaseComponent Members
      [Browsable(false)]
      public string Description
      {
         get
         {
            return "BizTalk Receive Pipeline Component for Encoding";
         }
      }
      
      [Browsable(false)]
      public string Name
      {
         get
         {
            return "Encode Pipeline Component";
         }
      }
      
      [Browsable(false)]
      public string Version
      {
         get
         {
            return "1.0";
         }
      }

      #endregion

      #region IComponent Members
      /// <summary>
      /// Method called during runtime
      /// </summary>
      /// <param name="pContext">Pipeline context</param>
      /// <param name="pInMsg">Message</param>
      /// <returns>Message with code word replacements</returns>
      public IBaseMessage Execute(IPipelineContext pContext, IBaseMessage pInMsg)
      {
         // Load the body of the message into an XmlDocument, find
         // the Location element, and replace the value as needed
         XmlDocument bodyDoc = new XmlDocument();
         IBaseMessagePart bodyPart = pInMsg.BodyPart;
         if (bodyPart != null)
         {
            try
            {
               bodyDoc.Load(bodyPart.Data);
            }
            catch (XmlException ex)
            {
               throw new ApplicationException(ex.Message);
            }
            try 
            {
               // find the Activity element
               XmlNamespaceManager nsmgr = new XmlNamespaceManager(bodyDoc.NameTable);
               nsmgr.AddNamespace("ns0", "http://Pipeline_Chapter_Project.Activity");
               XmlNode locNode = bodyDoc.DocumentElement.SelectSingleNode("/ns0:Activity/Location", nsmgr);
               string val = locNode.InnerText;
               switch (val)
               {
                  case "New York":
                     locNode.InnerText = "Homeplate";
                     break;
                  case "Philadelphia":
                     locNode.InnerText = "Pooltable";
                     break;
                  case "Chicago":
                     locNode.InnerText = "Thunderball";
                     break;
               }
            }
            catch (XPathException ex)
            {
               throw new ApplicationException("Probable wrong document type: " + ex.Message);
            }
            // rewind the stream and write the possibly changed document back out
            try 
            {
               Stream strm = new MemoryStream();
               bodyDoc.Save(strm);
               strm.Position = 0; // rewind or no one can see your data
               bodyPart.Data = strm;

               pContext.ResourceTracker.AddResource(strm);
            }
            catch (Exception ex)
            {
               throw new ApplicationException("Error trying to write encoded doc: " + ex.Message);
            }
         }
         return pInMsg;
      }

      #endregion

      #region IPersistPropertyBag Members

      public void InitNew()
      {
      }
      // used when accessing the component from unmanaged code
      public void GetClassID(out Guid classID)
      {
          classID = new Guid ("ACC3F15A-C389-4a5d-8F8E-2A951CDC4C19");
      }

      public void Load(IPropertyBag propertyBag, int errorLog)
      {
         object val = null;
         try
         {
            propertyBag.Read("ConnectionString", out val, 0);
         }
         catch (ArgumentException)
         {
         }
         catch (Exception ex)
         {
            throw new ApplicationException("Error reading propertybag: " + ex.Message);
         }

         if (val != null)
            connectString = (string)val;
         else
            connectString = "";
      }

      public void Save(IPropertyBag propertyBag, bool clearDirty, bool saveAllProperties)
      {
         object val = (object)ConnectionString;
         propertyBag.Write("ConnectionString", ref val);
      }

      #endregion

      #region IComponentUI Members
      /// <summary>
      /// Validate is called by the pipeline editor during the build process.
      /// If we were actually using a database of code words - replacement pairs, this
      /// is where we could do validity or connection testing.
      /// </summary>
      /// <param name="projectSystem"></param>
      /// <returns></returns>
      public IEnumerator Validate(object projectSystem)
      {
         return null;
      }
      /// <summary>
      /// Return a handle to a loaded icon bitmap
      /// BizTalk's toolbox provides a default icon, so we'll skip this.
      /// </summary>
      /// 
      [Browsable(false)]
      public System.IntPtr Icon
      {
         get
         {
            return ((Bitmap)resManager.GetObject("IconBitmap")).GetHicon();
         }
      }

      #endregion
   }
}
